
										// **** Image includes
#ifndef IMAGE_DEF
	#define	IMAGE_DEF
	
	#include <stdio.h>
	#include <stdlib.h>
	#include <conio.h>

	#include "common.h"

	#ifndef XBYTE
		#define XCHAR				unsigned char
		#define XBYTE				unsigned __int8
		#define XBYTE2				unsigned __int16
		#define XBYTE4				unsigned __int32
		#define XBYTE8				__int64
	#endif	
										// **** Image Pixel-Format Information (bit flags)
	#define IMG_UNDEF			0		// Image information is undefined.
	#define IMG_BW				1		// Data contains a B & W image (BPP = 1)
	#define IMG_COLOR			2		// Data contains a color image (RGB)
	#define IMG_COLOR_INDEX		4		// Data contains 8-bit index image (BPP=8)
	#define IMG_COLOR_LOW		8		// Data contains 12-bit color image (BPP=4-4-4)
	#define IMG_COLOR_HIGH		16		// Data contains 16-bit color image (BPP=6-5-5)
	#define IMG_COLOR_TRUE		32		// Data contains 24-bit color image (BPP=8-8-8)
	#define IMG_COLOR_MASKED	64		// Color data uses 0-value as mask
	#define IMG_ALPHA_MERG		128		// Data contains an alpha channel (A)
	#define IMG_ALPHA_LOW		256		// Alpha channel is 4-bit data	
	#define IMG_ALPHA_HIGH		512		// Alpha channel is 8-bit data
	#define IMG_ALPHA_SEP		1024	// Alpha channel data is separate
	#define IMG_DEPTH_MERG		2048	// Data contains depth data (D)
	#define IMG_DEPTH_SEP		4096	// Depth channel data is separate
	
	// NOTE: Based on the above flags, the following image formats are 
	// possible (where and _ underline indicates seperate memory spaces):
	//		I
	//		RGB		RGBA	RGB_A
	//		RGBD	RGBAD	RGBD_A
	//		RGB_D	RGBA_D	RGB_A_D

	// Shortcuts
	#define IMG_RGB			(IMG_COLOR_TRUE)
	#define IMG_RGBA		(IMG_COLOR_TRUE | IMG_ALPHA_MERG)
	#define IMG_RGBAD		(IMG_COLOR_TRUE | IMG_ALPHA_MERG | IMG_DEPTH_MERG)

										// **** Image Status Flags
	#define IMG_NOT_READY		0
	#define IMG_UPDATED			1
	#define IMG_READY			2

										// **** File Formats
	#define IMG_RAW_FORMAT		0		// RAW format
	#define IMG_TIFF_FORMAT		1		// TIFF format	

	#define DEGtoRAD			(3.141592/180)

	typedef unsigned char		ImageOps;



	class Image {
	public:		
		// Construction/Destruction
		Image ();									// Constructors
		Image (int xr, int yr, ImageOps ops);
		~Image ();									// Destructor
		void Size (int xr, int yr, ImageOps ops);	// Construct Image
		
		// Image Operations
		void Fill (XBYTE r, XBYTE g, XBYTE b);
		void SetPixel (int x, int y, XBYTE r, XBYTE g, XBYTE b);
		void SetPixel (int x, int y, XBYTE r, XBYTE g, XBYTE b, XBYTE a);
		void CopyPixels (int xoff, int yoff, Image& dest);
		
		// Power-Image Functions
		void ConvertToPowerImage (void);	

		// Image State Functions
		void SetStatus (XBYTE s)	{m_status = s;}
		void SetGLObject (int obj)	{m_GLobj = obj; m_status = IMG_READY;}
		void SetGLList (int list)	{m_GLlist = list;}
		void SetPivot (int x, int y)	{m_xpiv = (float) x; m_ypiv = (float) y;}	

		int GetWidth (void)		{return m_xres;}
		int GetHeight (void)	{return m_yres;}
		int GetSubWidth (void)	{return m_xsub;}
		int GetSubHeight (void)	{return m_ysub;}
		float GetSubX (void)	{return (float) m_xsub / m_xres;}
		float GetSubY (void)	{return (float) m_ysub / m_yres;}		
		unsigned char GetBitsPerPixel (void);		// Get Bits-Per-Pixel		
		XBYTE* GetData (void) {return m_data;}
		void GetPowerSize (int &x, int &y);
		XBYTE GetStatus (void)		{return m_status;}
		int GetGLObject (void)		{return m_GLobj;}
		int GetGLList (void)		{return m_GLlist;}
		int GetGLFormat (void);
		float GetPivotX (void)		{return m_xpiv;}
		float GetPivotY (void)		{return m_ypiv;}

		bool HasAlpha (void);	
		bool HasAlphaMerged (void);
		bool HasAlphaSeparate (void);
		bool HasDepth (void);
		bool HasDepthMerged (void);
		bool HasDepthSeparate (void);
		bool HasPowerSize (void);
						
		//void CreatePowerImage (Image &src);		
		/*void TurnOn (ImageOps opt);					// Turn on a PixelFormat option
		void TurnOff (ImageOps opt);					// Turn off a PixelFormat option	
		 
		int GetFormat (ImageOps ops);				// Get Format 
		int GetAlpha (ImageOps ops);					// Get Alpha
		char GetUsage (void)	{return m_usage;}
		void SetUsage (int u) {m_usage = u;}
		XBYTE* GetData (void)	{return m_data;}
		int GetHeight (void)		{return m_xres;}
		int GetWidth (void)		{return m_yres;}
		int GetIntensity (int x, int y);
		void ConvertFormat (ImageOps pf);
		void ConvertAlphaToMask (void);
		void ConvertAlphaToBlend (void);
		void UseWith (int what);

		void Rotate (ImageX *dest, float ang, float scale);
		void RotateImage (ImageX *dest, float ang, float scale);
		void RotateAlpha (ImageX *dest, float ang, float scale);
		*/

		

		XBYTE			m_status;						// Image Status
		int				m_GLobj;						// Image GL object (for drawing)
		int				m_GLlist;						// Image GL list (for drawing)
		ImageOps		m_options;						// Image Pixel Format
		int				m_xres, m_yres;					// Image Resolution
		int				m_xsub, m_ysub;					// Image Sub-Power Resolution
		int				m_size;
		float			m_xpiv, m_ypiv;					// Pivot Position
		XBYTE			*m_data;						// Color Data
		XBYTE			*m_alpha;						// Alpha Channel
		float			*m_depth;						// Depth Channel
		
		//char status;										// Image Ready?
	};

	static int GetSmallestPower (int x);
	static bool ClipPixels (int src_wid, int src_hgt, int dest_wid, int dest_hgt, int xoff, int yoff, int vx1, int vy1, int vx2, int vy2, int &src_offs, int &src_xs, int &src_ys, int &dest_offs);
	static bool ClipTile (int x, int y, int xres, int yres, int &mx, int &my, int &offx, int &offy);
	static bool TransferPixels (XBYTE* src, int src_wid, int src_hgt, ImageOps src_ops, XBYTE* dest, int dest_wid, int dest_hgt, ImageOps dest_ops, int xoff, int yoff);	
#endif
