

#ifndef NET_ENG
	#define NET_ENG

	#include "buffer.h"
	#include "netsock.h"

	#define NET_BUFSIZE			8000

	#define NET_NO_MODE			0
	#define NET_CLIENT			1
	#define NET_SERVER			2
	
	#define NET_OFF				0
	#define NET_START			1
	#define NET_RESET			2
	#define NET_DISCONNECTED	3
	#define NET_SEARCHING		4
	#define NET_CONNECTING		5
	#define NET_CONNECTED		6
	#define NET_PROCESSING		7

	#define NET_UDP				0

	#define NET_BROADCAST_OFF	0
	#define NET_BROADCAST_ON	1

	#define NET_SEARCH_PORT		8000

	#define SERV_IN				0
	#define CLIN_IN				1

	class NetData {
	public:
		void AttachInt (int i)		{* (int*) dat = i; dat += sizeof(int); dat_len += sizeof(int);}
		void AttachFloat (float f)	{* (float*) dat = f; dat += sizeof(float); dat_len += sizeof(float);}
		void AttachDouble (double d)	{* (double*) dat = d; dat += sizeof(double); dat_len += sizeof(double);}
		void AttachBool (bool b)	{* (bool*) dat = b; dat += sizeof(bool); dat_len += sizeof(bool);}
		int RetrieveInt (void)		{int i = * (int*) dat; dat += sizeof(int); dat_len -= sizeof(int); return i;}
		float RetrieveFloat (void)	{float f = * (float*) dat; dat += sizeof(float); dat_len -= sizeof(float); return f;}
		double RetrieveDouble (void)	{double d = * (double*) dat; dat += sizeof(double); dat_len -= sizeof(double); return d;}
		bool RetrieveBool (void)	{bool b = * (bool*) dat; dat += sizeof(bool); dat_len -= sizeof(bool); return b;}
		char	*dat;
		int		dat_len;
	};

	class NetEng {
	public:
		NetEng();
		// User functions
		void	Start (char m);
		void	Go (void);
		
		// State functions
		void	ClientStart (void);
		void	ClientReset (void);
		void	ClientDisconnected (void);
		void	ClientSearching (void);
		void	ClientConnecting (void);
		void	ClientConnected (void);
		
		void	ServerStart (void);
		void	ServerReset (void);
		void	ServerDisconnected (void);
		void	ServerProcessing (void);

		// Helper functions
		int		AddSocket (char protocol);
		int		FindSocketWithSrcPort (int chk_port);
			
		inline void SetSocketDest (int sn, NetAddress &d)	{net_sockets[sn].SetDest (d);}
		inline void SetSocketDestPort (int sn, int p)		{net_sockets[sn].SetDestPort (p);}
		inline void	SetSocketDestBroadcast (int sn, int p)	{net_sockets[sn].SetDestBroadcast (p);}
		inline NetAddress &GetSocketDest (int sn)			{return net_sockets[sn].GetDest ();}
		inline void GetSocketDestName (int sn, char* buf, unsigned long* buflen)	{net_sockets[sn].GetDestName (buf, buflen);}
		
		inline void SetSocketSrc (int sn, NetAddress &d)	{net_sockets[sn].SetSrc (d);}
		inline void SetSocketSrcPort (int sn, int p)		{net_sockets[sn].SetSrcPort (p);}
		inline void SetSocketSrcAny (int sn, int p)			{net_sockets[sn].SetSrcAny (p);}
		inline void SetSocketSrcToDest (int sn1, int sn2)	{net_sockets[sn2].SetDest (net_sockets[sn1].GetSrc());}
		inline NetAddress &GetSocketSrc (int sn)			{return net_sockets[sn].GetSrc ();}		
		inline void GetSocketSrcName (int sn, char* buf, unsigned long* buflen)	{net_sockets[sn].GetSrcName (buf, buflen);}
		
		inline void	SetSocketMode (int sn, char m)			{net_sockets[sn].SetMode (m);}
		inline void SetSocketTimeout (int sn, int sec, int msec) {net_sockets[sn].SetTimeout (sec, msec);}
		inline void SetSocketSendTime (int sn, int t)		{net_sockets[sn].SetSendTime (t);}
		inline int GetSocketSendTime (int sn)				{return net_sockets[sn].GetSendTime ();}
		
		inline int SocketSendDataTo (int sn, char *dat, int len)	{return net_sockets[sn].SendDataTo (dat, len);}
		inline int SocketRecvDataFrom (int sn, char *dat, int len) {return net_sockets[sn].RecvDataFrom (dat, len);}
		inline int SocketSendData (int sn, char *dat, int len)	{return net_sockets[sn].SendData (dat, len);}
		inline int SocketRecvData (int sn, char *dat, int len) {return net_sockets[sn].RecvData (dat, len);}
		inline void SocketBindToSrc (int sn)				{net_sockets[sn].BindToSrc ();}
		inline void SocketBindToDest (int sn)				{net_sockets[sn].BindToDest ();}
		inline void SocketConnectToSrc (int sn)				{net_sockets[sn].ConnectToSrc ();}
		inline void SocketConnectToDest (int sn)			{net_sockets[sn].ConnectToDest ();}

		inline void SetSocketConnected (int sn, bool c)		{net_sockets[sn].SetConnected (c);}
		inline bool GetSocketConnected (int sn)				{return net_sockets[sn].GetConnected ();}

		int Recieve (void);		
		int Send (int sn, char* buf, int len);
	
		int GetNumLines (void)		{return output.GetNumLines();}
		char* GetLine (int n)		{return output.GetLine (n);}

		int GetNumClients (void)	{return num_net_sockets-1;}
		int GetClientNumber (void)	{return client_sn-1;}

		void StartDatagram (void);
		NetData* StartMessage (void);
		void StopMessage (void);
		void SendDatagram (int s);

		NetData* RecvMessage (void);

	//	unsigned int GetClientNumber (void)	{return client_sn;}
	//	int GetNumClients (void)			{return num_net_sockets-1;}
	private:
		char			mode;
		char			state;
		int				max_net_sockets;
		int				num_net_sockets;
		NetSock*		net_sockets;
		struct fd_set	socket_set;
		unsigned int	serv_port;
		int				client_sn;

		int				curr_socket;

		int				buf_len;
		char*			buf_pos;
		char			buf[NET_BUFSIZE];
		NetData			buf_data;

		
		//Buffer			data;
		//Buffer			server_data;
		Buffer			output;
	};

#endif
