
#include <time.h>
#include <winsock2.h>
#include <stdio.h>

#include "neteng.h"
#include "netsock.h"
	
//server_in_addr.S_un.S_addr = htonl (0xC0A8002A);

void NetAddress::SetPort (int p)
{
	port = p;
	addr.sin_port = htons(port);
}

void NetAddress::SetBroadcast (void)
{
	in_addr.S_un.S_addr = htonl(INADDR_BROADCAST);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr = in_addr;
	memset (addr.sin_zero, 0, sizeof (addr.sin_zero));
}

void NetAddress::SetAny (void)
{
	in_addr.S_un.S_addr = htonl(INADDR_ANY);
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	addr.sin_addr = in_addr;
	memset (addr.sin_zero, 0, sizeof (addr.sin_zero));
}
	
void NetSock::SetMode (char m)
{
	bool optval;
	if (m==NET_BROADCAST_ON)	optval = 1;
	else						optval = 0;
	setsockopt (sock, SOL_SOCKET, SO_BROADCAST,  (const char*) &optval, sizeof(optval));

	unsigned long ioval = 1;
	ioctlsocket (sock, FIONBIO, &ioval);	 
}

void NetSock::Create (char p)
{
	sock = socket (AF_INET, SOCK_DGRAM, 0);
	connected = false;
	if (sock == INVALID_SOCKET) {
		printf ("Very bad! Cannot create socket.\n");
		exit (-1);
	}
}

int NetSock::SendDataTo (char *buf, int len)
{
	int result;
	dest.SetAddressSize();
	result = sendto (sock, buf, len, 0, dest.GetAddress(), dest.GetAddressSize());
	sendtime = clock();
	if (result==SOCKET_ERROR) {
		printf ("Error sending: %d\n", WSAGetLastError());
	} else {
		//printf ("Sent %d bytes (time: %d)\n", result, sendtime);
	}
	return result;
}

int NetSock::SendData (char *buf, int len)
{
	int result = send (sock, buf, len, 0);
	sendtime = clock();
	if (result==SOCKET_ERROR) {
		printf ("Error sending: %d\n", WSAGetLastError());
	}
	return result;
}

int NetSock::RecvDataFrom (char *buf, int len)
{
	int result;
	
	src.SetAddressSize ();
	result = recvfrom (sock, buf, len, 0, src.GetAddress(), src.GetAddressSizeMdfy ());
	recvtime = clock();
	if (result==SOCKET_ERROR) {
		printf ("Error recieving: %d\n", WSAGetLastError());
	} else {
		//printf ("Recieved %d bytes (time: %d)\n", result, recvtime);
	}
	return result;
}

int NetSock::RecvData (char *buf, int len)
{
	int result = recv (sock, buf, len, 0);
	recvtime = clock();
	if (result==SOCKET_ERROR) {
		printf ("Error recieving: %d\n", WSAGetLastError());
	}
	return result;
}


int NetSock::BindToSrc (void)
{
	int result;	
	src.SetAddressSize ();
	result = bind (sock, src.GetAddress(), src.GetAddressSize());
	if (result==SOCKET_ERROR) 
		printf ("Error binding: %d\n", WSAGetLastError());
	return result;
}

int NetSock::BindToDest (void)
{
	int result;	
	dest.SetAddressSize ();
	result = bind (sock, dest.GetAddress(), dest.GetAddressSize());
	if (result==SOCKET_ERROR) 
		printf ("Error binding: %d\n", WSAGetLastError());
	return result;
}


int NetSock::ConnectToSrc (void)
{
	int result;
	src.SetAddressSize ();
	result = connect (sock, src.GetAddress(), src.GetAddressSize());	
	if (result==SOCKET_ERROR) 
		printf ("Error connecting: %d\n", WSAGetLastError());
	connected = true;
	return result;
}

int NetSock::ConnectToDest (void)
{
	int result;
	dest.SetAddressSize ();
	result = connect (sock, dest.GetAddress(), dest.GetAddressSize());
	if (result==SOCKET_ERROR) 
		printf ("Error connecting: %d\n", WSAGetLastError());
	return result;
}