
#include <math.h>

#include "neteng.h"
#include "win.h"
#include "shipnet-eng.h"

void ShipEng::Initialize (bool i_am_client, NetEng* eng)
{
	ImageExt imgext;
	char tiffile[100];
	int n;

	m_particles.Initialize();					
	for (n=0; n < MAX_SHIPS; n++)
		m_ship[n].Initialize(n);		

	for (n=0; n < 4; n++) {
		sprintf (tiffile, "Images\\shield%02d.tif", n);
		imgext.Load (tiffile, &m_ship_shields[n]);
		m_ship_shields[n].SetPivot (50, 50);
	}
	
	imgext.Load ("Images\\moon.tif", &m_back_img);
	
	m_neteng = eng;	
	
	m_getname[0] = '\0';
	m_getname_pos = m_getname;

	m_client_ship = -1;
	m_update_data = 0;	

	m_photon_sound.LoadWav ("photon.wav");
	m_engine_sound.LoadWav ("engines.wav");
	m_engineother_sound.LoadWav ("engines.wav");
}

void ShipEng::ClientGo (void)
{
	switch (m_state) {
	case STATE_NEEDNAME: {
		m_neteng->Go ();
		char key = WinGL.GetKey ();
		if (key!=0) {
			if (key==13)	m_state = STATE_PLAY;
			else			*m_getname_pos++ = key;
			*m_getname_pos = '\0';				
		}
	
		WinGL.DrawOpen2D ();
		char disp[100];
		sprintf (disp, "Enter name: %s", m_getname);
		WinGL.DrawFill (0,0, 1024, 768, 0.0, 0.0, 0.0, 1.0);
		WinGL.DrawColor (255, 255, 255);
		WinGL.DrawText (20, 40, disp);
		WinGL.DrawClose ();
	} break;
	case STATE_PLAY: {		
		for (int n=0; n < 50; n++) {
			m_neteng->Go ();
			if (m_client_ship!=-1) ClientRecieve ();
		}		
		if (m_client_ship==-1) {
			m_client_ship = m_neteng->GetClientNumber ();
			if (m_client_ship!=-1) {
				m_ship[m_client_ship].SetName (m_getname);
				ClientSendShipNameMsg ();				
			}
		} else {
			if (m_frame % LATENCY == 0) ClientSendShipMoveMsg ();

			//if (m_frame % 30 == 0) WinGL.PlaySound (&m_engine_sound, PLAY_REWIND, 0.2, 0.0);

			for (n=0; n < m_num_ships; n++)
				m_ship[n].Move();
			m_particles.Move();

			ClientProcessInput ();

			ClientDetectCollisions ();
			
			float csx = m_ship[m_client_ship].m_x;
			float csy = m_ship[m_client_ship].m_y;

			WinGL.DrawOpen2D ();			
			WinGL.DrawBackground (m_back_img, (int) csx, (int) csy);
			for (n=0; n < m_num_ships; n++)
				m_ship[n].Draw (csx, csy, m_frame, m_ship_shields);
			m_particles.Draw (csx, csy, m_frame);
		
			
			/*char score[100];
			for (n=0; n < m_num_ships; n++) {
			
			  sprintf (score, "%s: %d", m_ship[n].name, m_ship[n].m_frags);
				WinGL.DrawText (10, 20*n+10, score);
			}*/
		}		
		WinGL.DrawClose ();	
		m_frame++;
	} break;
	}
}

void ShipEng::ClientProcessInput (void)
{
	bool l, r, u, d, sl, sr;

	if (WinGL.GetKeyDown (VK_SPACE)) {	
		float x, y, dx, dy, photon_a;
		
		for (int j=-4; j < 4; j++) {
			x = m_ship[m_client_ship].m_x;
			y = m_ship[m_client_ship].m_y;
			photon_a = m_ship[m_client_ship].m_turn_a + j;
			dx = m_ship[m_client_ship].m_dx + float(sin(photon_a*DEGtoRAD)*30.0f);
			dy = m_ship[m_client_ship].m_dy - float(cos(photon_a*DEGtoRAD)*30.0f);
			ClientSendPhotonMsg (x, y, dx, dy);
			m_particles.AddPhoton (m_client_ship, x, y, dx, dy);			
		}

		WinGL.PlaySound (&m_photon_sound, PLAY_REWIND, 0.9, 0.0);		
	}
	l = WinGL.GetKeyDown (VK_LEFT);
	r = WinGL.GetKeyDown (VK_RIGHT);
	u = WinGL.GetKeyDown (VK_UP);
	d = WinGL.GetKeyDown (VK_DOWN);
	sl = false;
	sr = false;
	m_ship[m_client_ship].UpdateShipByInput (l, r, u, d, sl, sr);
	
}


void ShipEng::ClientRecieve (void)
{
	int msg;
	NetData* dat;
	dat = m_neteng->RecvMessage ();
	while (dat->dat_len > 0) {
		msg = dat->RetrieveInt ();		
		switch (msg) {
		case SHIP_UPDATE_MSG:	ClientRecvShipUpdateMsg (dat); break;
		case SHIP_NAME_MSG:		ClientRecvShipNameMsg (dat); return; break;
		case PHOTON_MSG:		ClientRecvPhotonMsg (dat); return; break;		
		case COLLISION_MSG:		ClientRecvCollisionMsg (dat); break;
		}
		dat = m_neteng->RecvMessage ();
	}
}

void ShipEng::ClientRecvShipUpdateMsg (NetData* dat)
{
	int msg_frame, n;
	int last_send_frame;
	float dt, x, y, dx, dy, va, sp, ta, tda;
	msg_frame = dat->RetrieveInt ();
	m_num_ships = dat->RetrieveInt ();
	
	if (msg_frame > m_last_update_frame) {
		m_last_update_frame = msg_frame;
		for (n=0; n < m_num_ships; n++) {
			last_send_frame = dat->RetrieveInt ();
			dt = dat->RetrieveFloat ();
			x = dat->RetrieveFloat ();
			y = dat->RetrieveFloat ();
			dx = dat->RetrieveFloat ();
			dy = dat->RetrieveFloat ();
			va = dat->RetrieveFloat ();
			sp = dat->RetrieveFloat ();
			ta = dat->RetrieveFloat ();
			tda = dat->RetrieveFloat ();
			if (n==m_client_ship) {
				m_ship[n].SetDT ((m_frame-last_send_frame)*0.5f);				
			} else {
				m_ship[n].UpdateShipByData (dt, x, y, dx, dy, va, sp, ta, tda);				
			}
		}
	}
}

void ShipEng::ClientRecvShipNameMsg (NetData* dat)
{
	int msg_num_ships, msg_ship, msg_len;
	char buf[100];
	int n, j;

	msg_num_ships = dat->RetrieveInt ();
	for (j=0; j < msg_num_ships; j++) {
		msg_ship = dat->RetrieveInt ();
		msg_len = dat->RetrieveInt ();
		for (n=0; n < msg_len; n++)
			buf[n] = (char) dat->RetrieveInt ();
		m_ship[msg_ship].SetName (buf);
	}
}


void ShipEng::ClientRecvPhotonMsg (NetData* dat)
{
	int msg_ship;
	float x, y, dx, dy;
	float sx, sy, sdx, sdy, sva, ssp, sta, stda;
	msg_ship = dat->RetrieveInt ();
	x = dat->RetrieveFloat ();
	y = dat->RetrieveFloat ();
	dx = dat->RetrieveFloat ();
	dy = dat->RetrieveFloat ();
	sx = dat->RetrieveFloat ();
	sy = dat->RetrieveFloat ();	
	sdx = dat->RetrieveFloat ();
	sdy = dat->RetrieveFloat ();
	sva = dat->RetrieveFloat ();
	ssp = dat->RetrieveFloat ();
	sta = dat->RetrieveFloat ();
	stda = dat->RetrieveFloat ();
	if (msg_ship!=m_client_ship) {
		m_particles.AddPhoton (msg_ship, x, y, dx, dy);
		m_ship[msg_ship].UpdateShipByData (1.0, sx, sy, sdx, sdy, sva, ssp, sta, stda);
	}
}

void ShipEng::ClientRecvCollisionMsg (NetData* dat)
{
	int a, b;
	float ax, ay, adx, ady, av, as, bx, by, bdx, bdy, bv, bs;
	
	a = dat->RetrieveInt ();
	ax = dat->RetrieveFloat();
	ay = dat->RetrieveFloat();
	adx = dat->RetrieveFloat();
	ady = dat->RetrieveFloat();
	av = dat->RetrieveFloat();
	as = dat->RetrieveFloat();
	b = dat->RetrieveInt ();
	bx = dat->RetrieveFloat();
	by = dat->RetrieveFloat();
	bdx = dat->RetrieveFloat();
	bdy = dat->RetrieveFloat();
	bv = dat->RetrieveFloat();
	bs = dat->RetrieveFloat();
	m_ship[a].UpdateShipByData (ax, ay, adx, ady, av, as);
	m_ship[b].UpdateShipByData (bx, by, bdx, bdy, bv, bs);
}

void ShipEng::ClientSendShipNameMsg (void)
{
	NetData* dat;
	char buf[100];
	int n;	
	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (SHIP_NAME_MSG);
	strcpy (buf, m_ship[m_client_ship].GetName());
	dat->AttachInt (m_client_ship);
	dat->AttachInt (strlen(buf)+1);
	for (n=0; n < strlen(buf)+1; n++)
		dat->AttachInt ((int) buf[n]);
	
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (0);
}

void ShipEng::ClientSendShipMoveMsg (void)
{
	NetData* dat;
	m_neteng->StartDatagram();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (SHIP_MOVE_MSG);
	dat->AttachInt (m_client_ship);
	dat->AttachInt (m_frame);						// Send client time when packet sent
	dat->AttachFloat (m_ship[m_client_ship].m_dt);	// Send current guess at packet transmission time
	dat->AttachFloat (m_ship[m_client_ship].m_x);
	dat->AttachFloat (m_ship[m_client_ship].m_y);
	dat->AttachFloat (m_ship[m_client_ship].m_dx);
	dat->AttachFloat (m_ship[m_client_ship].m_dy);
	dat->AttachFloat (m_ship[m_client_ship].m_vel_a);
	dat->AttachFloat (m_ship[m_client_ship].m_speed);	
	dat->AttachFloat (m_ship[m_client_ship].m_turn_a);
	dat->AttachFloat (m_ship[m_client_ship].m_turn_da);	
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (0);
}

void ShipEng::ClientSendPhotonMsg (float x, float y, float dx, float dy)
{	
	NetData* dat;
	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (PHOTON_MSG);
	dat->AttachInt (m_client_ship);	
	dat->AttachFloat (x);
	dat->AttachFloat (y);
	dat->AttachFloat (dx);
	dat->AttachFloat (dy);
	dat->AttachFloat (m_ship[m_client_ship].m_x);
	dat->AttachFloat (m_ship[m_client_ship].m_y);	
	dat->AttachFloat (m_ship[m_client_ship].m_dx);
	dat->AttachFloat (m_ship[m_client_ship].m_dy);
	dat->AttachFloat (m_ship[m_client_ship].m_vel_a);
	dat->AttachFloat (m_ship[m_client_ship].m_speed);
	dat->AttachFloat (m_ship[m_client_ship].m_turn_a);
	dat->AttachFloat (m_ship[m_client_ship].m_turn_da);	
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (0);
}

void ShipEng::ClientSendCollisionMsg (int a, int b)
{
	NetData* dat;
	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (COLLISION_MSG);
	dat->AttachInt (a);	
	dat->AttachFloat (m_ship[a].m_x);
	dat->AttachFloat (m_ship[a].m_y);
	dat->AttachFloat (m_ship[a].m_dx);
	dat->AttachFloat (m_ship[a].m_dy);
	dat->AttachFloat (m_ship[a].m_vel_a);
	dat->AttachFloat (m_ship[a].m_speed);
	dat->AttachInt (b);
	dat->AttachFloat (m_ship[b].m_x);
	dat->AttachFloat (m_ship[b].m_y);
	dat->AttachFloat (m_ship[b].m_dx);
	dat->AttachFloat (m_ship[b].m_dy);
	dat->AttachFloat (m_ship[b].m_vel_a);
	dat->AttachFloat (m_ship[b].m_speed);
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (0);
}

void ShipEng::ServerGo (void)
{
	for (int n=0; n < 50; n++) {
		m_neteng->Go ();
		ServerRecieve ();
	}
	
	m_num_ships = m_neteng->GetNumClients ();
	if (m_frame % LATENCY == 0) ServerSendShipUpdateMsg ();
	
	WinGL.DrawOpen2D();
	WinGL.DrawFill (0,0,1024, 768, 0.0, 0.0, 0.0, 1.0);
	WinGL.DrawColor (255.0, 255.0, 255.0);
	DrawNetEng();
	WinGL.DrawClose();

	m_frame++;
}

void ShipEng::ServerRecieve (void)
{
	int msg;
	NetData* dat;
	dat = m_neteng->RecvMessage ();
	while (dat->dat_len > 0) {
		msg = dat->RetrieveInt ();		
		switch (msg) {
		case SHIP_MOVE_MSG:		ServerRecvShipMoveMsg (dat); break;
		case SHIP_NAME_MSG:		ServerFwdShipNameMsg (dat); return; break;
		case PHOTON_MSG:		ServerFwdPhotonMsg (dat); return; break;		
		case COLLISION_MSG:		ServerFwdCollisionMsg (dat); return; break;
		}
		dat = m_neteng->RecvMessage ();
	}
}

void ShipEng::ServerRecvShipMoveMsg (NetData* dat)
{
	int msg_ship, msg_time;
	float dt, x, y, dx, dy, va, sp, ta, tda;
	msg_ship = dat->RetrieveInt ();
	msg_time = dat->RetrieveInt ();
	dt = dat->RetrieveFloat ();
	x = dat->RetrieveFloat ();
	y = dat->RetrieveFloat ();
	dx = dat->RetrieveFloat ();
	dy = dat->RetrieveFloat ();
	va = dat->RetrieveFloat ();
	sp = dat->RetrieveFloat ();
	ta = dat->RetrieveFloat ();
	tda = dat->RetrieveFloat ();	
	m_ship[msg_ship].SetLastSendTime (msg_time);
	m_ship[msg_ship].UpdateShipByData (dt, x, y, dx, dy, va, sp, ta, tda);
}

void ShipEng::ServerFwdShipNameMsg (NetData* dat)
{
	char buf[100];
	int msg_ship, msg_len;
	int n, j;
	msg_ship = dat->RetrieveInt ();
	msg_len = dat->RetrieveInt();
	for (n=0; n < msg_len; n++) {
		buf[n] = (char) dat->RetrieveInt ();
	}
	m_ship[msg_ship].SetName (buf);
		
	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (SHIP_NAME_MSG);
	dat->AttachInt (m_num_ships);
	for (j=0; j < m_num_ships; j++) {				// Send name for all ships
		strcpy (buf, m_ship[j].GetName());
		dat->AttachInt (j);
		dat->AttachInt (strlen(buf)+1);
		for (n=0; n < strlen(buf)+1; n++) 		// Send name buffer
			dat->AttachInt ((int) buf[n]);
	}
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (-1);				// Forward to all clients
}

void ShipEng::ServerFwdPhotonMsg (NetData* dat)
{
	int msg_ship;
	float x, y, dx, dy;
	float sx, sy, sdx, sdy, sva, sp, sa, sda;
	msg_ship = dat->RetrieveInt ();
	x = dat->RetrieveFloat();
	y = dat->RetrieveFloat();
	dx = dat->RetrieveFloat();
	dy = dat->RetrieveFloat();
	sx = dat->RetrieveFloat ();
	sy = dat->RetrieveFloat ();
	sdx = dat->RetrieveFloat ();
	sdy = dat->RetrieveFloat ();
	sva= dat->RetrieveFloat ();
	sp = dat->RetrieveFloat ();
	sa = dat->RetrieveFloat ();
	sda = dat->RetrieveFloat ();

	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (PHOTON_MSG);
	dat->AttachInt (msg_ship);
	dat->AttachFloat (x);
	dat->AttachFloat (y);
	dat->AttachFloat (dx);
	dat->AttachFloat (dy);
	dat->AttachFloat (sx);
	dat->AttachFloat (sy);
	dat->AttachFloat (sdx);
	dat->AttachFloat (sdy);
	dat->AttachFloat (sva);
	dat->AttachFloat (sp);
	dat->AttachFloat (sa);
	dat->AttachFloat (sda);
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (-1);				// Forward to all clients
}

void ShipEng::ServerFwdCollisionMsg (NetData* dat)
{
	int a, b;
	float ax, ay, adx, ady, av, as, bx, by, bdx, bdy, bv, bs;
	a = dat->RetrieveInt ();
	ax = dat->RetrieveFloat();
	ay = dat->RetrieveFloat();
	adx = dat->RetrieveFloat();
	ady = dat->RetrieveFloat();
	av = dat->RetrieveFloat();
	as = dat->RetrieveFloat();
	b = dat->RetrieveInt ();
	bx = dat->RetrieveFloat();
	by = dat->RetrieveFloat();
	bdx = dat->RetrieveFloat();
	bdy = dat->RetrieveFloat();
	bv = dat->RetrieveFloat();
	bs = dat->RetrieveFloat();
	m_ship[a].UpdateShipByData (ax, ay, adx, ady, av, as);
	m_ship[b].UpdateShipByData (bx, by, bdx, bdy, bv, bs);

	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (COLLISION_MSG);	
	dat->AttachInt (a);	
	dat->AttachFloat (ax);
	dat->AttachFloat (ay);
	dat->AttachFloat (adx);
	dat->AttachFloat (ady);
	dat->AttachFloat (av);
	dat->AttachFloat (as);
	dat->AttachInt (b);
	dat->AttachFloat (bx);
	dat->AttachFloat (by);
	dat->AttachFloat (bdx);
	dat->AttachFloat (bdy);
	dat->AttachFloat (bv);
	dat->AttachFloat (bs);
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (-1);
}


void ShipEng::ServerSendShipUpdateMsg (void)
{
	NetData *dat;
	
	m_neteng->StartDatagram ();
	dat = m_neteng->StartMessage ();
	dat->AttachInt (SHIP_UPDATE_MSG);
	dat->AttachInt (m_frame);
	dat->AttachInt (m_num_ships);	
	for (int n=0; n < m_num_ships; n++) {
		dat->AttachInt (m_ship[n].last_send_time);
		dat->AttachFloat (m_ship[n].m_dt);
		dat->AttachFloat (m_ship[n].m_x);
		dat->AttachFloat (m_ship[n].m_y);
		dat->AttachFloat (m_ship[n].m_dx);
		dat->AttachFloat (m_ship[n].m_dy);
		dat->AttachFloat (m_ship[n].m_vel_a);
		dat->AttachFloat (m_ship[n].m_speed);	
		dat->AttachFloat (m_ship[n].m_turn_a);
		dat->AttachFloat (m_ship[n].m_turn_da);
	}
	m_neteng->StopMessage ();
	m_neteng->SendDatagram (-1);
}

void ShipEng::DrawNetEng (void)
{
	for (int n=0; n < m_neteng->GetNumLines(); n++) {
		WinGL.DrawText (10, n*15, m_neteng->GetLine (n));
	}
}

void ShipEng::ClientDetectCollisions (void)
{
	int i, j, cp;
	float deltx, delty, dist, speedi, speedj;
	float recoilx, recoily;

	for (i=0; i < m_num_ships; i++) {
		for (j=i+1; j < m_num_ships; j++) {			
			deltx = m_ship[i].m_x - m_ship[j].m_x;
			delty = m_ship[i].m_y - m_ship[j].m_y;
			dist = (float) sqrt (deltx*deltx + delty*delty);
			if (dist < 1200) {
				float vol = 1.0f - (dist/1200.0f);
				if (vol > 1.0f) vol = 1.0f;
				if (m_frame % 5 == 0) WinGL.PlaySound (&m_engineother_sound, PLAY_CONTINUE, (float) vol, 0.0);
			}
			if (dist < SHIP_SIZE+SHIP_SIZE) {
				speedi = (float) sqrt (m_ship[i].m_dx*m_ship[i].m_dx + m_ship[i].m_dy*m_ship[i].m_dy);
				speedj = (float) sqrt (m_ship[j].m_dx*m_ship[j].m_dx + m_ship[j].m_dy*m_ship[j].m_dy);
				
				recoilx = deltx/dist;
				recoily = delty/dist;
				
				m_ship[i].m_x -= m_ship[i].m_dx;
				m_ship[i].m_y -= m_ship[i].m_dy;
				m_ship[j].m_x -= m_ship[j].m_dx;
				m_ship[j].m_y -= m_ship[j].m_dy;
				
				m_ship[i].m_speed = speedj*0.25f;	// transfer kinetic energy
				m_ship[j].m_speed = speedi*0.25f;
				m_ship[i].m_dx += recoilx * m_ship[i].m_speed;
				m_ship[i].m_dy += recoily * m_ship[i].m_speed;
				m_ship[j].m_dx -= recoilx * m_ship[j].m_speed;
				m_ship[j].m_dy -= recoily * m_ship[j].m_speed;				
				m_ship[i].m_vel_a = (atan2 (m_ship[i].m_dy, m_ship[i].m_dx)/DEGtoRAD)+90.0;
				m_ship[j].m_vel_a = (atan2 (m_ship[j].m_dy, m_ship[j].m_dx)/DEGtoRAD)+90.0;

				ClientSendCollisionMsg (i, j);
			}
		}

		cp = m_particles.DetectCollisions (i, m_ship[i].m_x, m_ship[i].m_y, m_ship[i].m_dx, m_ship[i].m_dy, &dist);
		if (cp!=-1) {
			m_ship[i].ActivateShields (dist);
			m_ship[i].m_shield -= 10;
			if (m_ship[i].m_shield < 0) {
				m_ship[i].m_shield = 0;
				m_ship[i].m_health -= 10;
			}
			if (m_ship[i].m_health < 0) {				
				for (j=0; j < 10; j++) {
					deltx = float(rand()*80/RAND_MAX) - 40;
					delty = float(rand()*80/RAND_MAX) - 40;
					recoilx = float(rand()*3.0f/RAND_MAX) - 1.5f;
					recoily = float(rand()*3.0f/RAND_MAX) - 1.5f;
					m_particles.AddExplosion (i, m_ship[i].m_x+deltx, m_ship[i].m_y+delty, m_ship[i].m_dx+recoilx, m_ship[i].m_dy+recoily);
				}
				m_ship[i].m_health = 1000;
				m_ship[i].m_shield = 1000;
				m_ship[i].m_x = (rand()*10000.0f/RAND_MAX)-5000.0f;
				m_ship[i].m_y = (rand()*10000.0f/RAND_MAX)-5000.0f;
				m_ship[i].m_dx = 0.0;
				m_ship[i].m_dy = 0.0;				
			}
		}
	}

	
}

