//
// GameX
// Ship Demo - Photon Class Code
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 
#include <math.h>

#include "win.h"
#include "shipnet-particles.h"


// Photons - Initialize
// Load Photon TIF image and initialize list of particles.
void Photons::Initialize (void)
{
	//char tiffile[100];
	ImageExt imgext;

	max_photons = 2000;
	photon_list = new Photon[max_photons];
	
	imgext.Load ("Images\\photon1.tif", &photon_img[0]);
	imgext.Load ("Images\\photon2.tif", &photon_img[1]);
	imgext.Load ("Images\\photon3.tif", &photon_img[2]);
	photon_img[0].SetPivot (25, 25);
}

// Photons - AddPhoton
// Add a photon to the list of particles.
int Photons::AddPhoton (float new_x, float new_y, float new_dx, float new_dy)
{
	int new_photon;
	if (num_photons==max_photons) {		// If we've reached maximum, select an existing
										// particle at random to replace.
		new_photon = (rand() * (max_photons-1))/RAND_MAX;
	} else {							
		new_photon = num_photons;		// If there is still allocated memory left,
		num_photons++;					// add new particle at the end
	}
	photon_list[new_photon].x = new_x;	// Initialize new photon with given parameters
	photon_list[new_photon].y = new_y;
	photon_list[new_photon].dx = new_dx;
	photon_list[new_photon].dy = new_dy;
	photon_list[new_photon].phase = (rand()*360)/RAND_MAX;
	photon_list[new_photon].age = 0;	
	photon_list[new_photon].img = 0;
	return new_photon;					// Return the number of the photon just added
}

// Photons - Move
// Move all the Photons in the particle system.
// The Photons are implemented as a very simple physics engine for 
// a particle system. Each particle is advanced according to its velocity.
// Particles out of bounds are rebounded into the view.
// Particles which are too old are deleted.
void Photons::Move (void)
{ 
										// Process all particles (photons) for moving
	for (int n=0; n<num_photons; n++) {	
		photon_list[n].x += photon_list[n].dx;	// Move particles by velocity
		photon_list[n].y += photon_list[n].dy;
												// Rebound particles off view edges
		photon_list[n].age++;					// Increment age of particle
		if (photon_list[n].age>400) {			// If particle is too old..
			photon_list[n].x = photon_list[num_photons-1].x;	// Delete it putting the
			photon_list[n].y = photon_list[num_photons-1].y;	// last one in its place.
			photon_list[n].dx = photon_list[num_photons-1].dx;
			photon_list[n].dy = photon_list[num_photons-1].dy;
			photon_list[n].age = photon_list[num_photons-1].age;
			photon_list[n].phase = photon_list[num_photons-1].phase;
			num_photons--;										// Decrement total number.
		}
	}
}

// Photons - Draw
// Draw all Photons
void Photons::Draw (float dx, float dy, int frame)
{
	float i, s;					// Process all particles (photons) for drawing
	int cx, cy;

	cx = WinGL.GetWinX()/2;
	cy = WinGL.GetWinY()/2;

	for (int n=0; n<num_photons; n++) {
		i = (rand()*360.0f)/RAND_MAX;
										// Draw Photon on the screen using GameX
		s = 0.7f + sin((frame+photon_list[n].phase)*12*3.141592/180)*0.3f;
		WinGL.DrawImage (photon_img[0], (int) photon_list[n].x-dx+cx, (int) photon_list[n].y-dy+cy, i, s);
	}
}
