//
// GameX
// Ship Demo - Ship Class Code
// 
// Copyright (C) 2002 Rama C. Hoetzlein
// 
// This software is released under the GameX GNU GPL 
// Open Source Liscense. See the GameX documentation included
// with this source code for terms of modification, 
// distribution and re-release. 
// 

#include <math.h>

#include "win.h"
#include "shipnet-ship.h"

// Ship - Initialize
// Initialize all ship images, ship shadow and ship's target image
void Ship::Initialize (int num)
{
	char tiffile[100];
	ImageExt imgext;
	int n;
	int ship_clr;

	m_ship_num = num;									// Initialize ship number													

	for (n=SHIP_IMG_LEFT; n<=SHIP_IMG_RIGHT; n++) {	// Load all ship images
		ship_clr = (m_ship_num % 3) + 1;
		sprintf (tiffile, "Images\\ship-%04d-%02d.tif", n, ship_clr);
		imgext.Load (tiffile, &ships[n]);
		ships[n].SetPivot (40, 40);
	}
													// Load ship's shadow image
	imgext.Load ("Images\\ship-shadow.tif", &ships[SHIP_IMG_SHADOW]);
	ships[SHIP_IMG_SHADOW].SetPivot (40, 40);

													// Load ship's target image
	imgext.Load ("Images\\target.tif", &target);
	
	m_curr_img = SHIP_IMG_FLAT;						// Set current ship image

	m_refire = 0;	
	m_x = float((rand()*10000.0f/RAND_MAX)-5000.0f);
	m_y = float((rand()*10000.0f/RAND_MAX)-5000.0f);
	m_turn_a = 0.0;
	m_vel_a = 0.0;
	m_speed = 0.0;

	m_health = 1000;
	m_shield = 1000;
	m_shield_on = -1;
	m_shield_ang = 0.0;
	m_dx = 0.0;
	m_dy = 0.0;

	m_frags = 0;
}


void Ship::UpdateShipByInput (bool l, bool r, bool u, bool d, bool sl, bool sr)
{
	float dxl = m_dx;
	float dyl = m_dy;
	
	bool afterburn = WinGL.GetKeyDown (VK_TAB);

	if (l) {
		if (WinGL.GetKeyDown(VK_CONTROL)) {
			m_vel_a -= 2.0;
			m_turn_a = m_vel_a;
			m_dx = sin(m_vel_a*DEGtoRAD) * m_speed;
			m_dy = -cos(m_vel_a*DEGtoRAD) * m_speed;			
		} else {
			m_turn_da -= 0.5f;
		} 
		m_curr_img++;						// Rotate ship left
		if (m_curr_img>16) m_curr_img = 16;
	} else if (r) {
		if (WinGL.GetKeyDown(VK_CONTROL)) {
			m_vel_a += 2.0;
			m_turn_a = m_vel_a;
			m_dx = sin(m_vel_a*DEGtoRAD) * m_speed;
			m_dy = -cos(m_vel_a*DEGtoRAD) * m_speed;			
		} else {
			m_turn_da += 0.5f;	
		}
		m_curr_img--;						// Rotate ship right
		if (m_curr_img<0) m_curr_img = 0;
	} else {		
		if (m_turn_da > 0) m_turn_da /= 2;					// Gradually stop the ship from
		if (m_turn_da < 0) m_turn_da /= 2;					// rotating
		if (m_curr_img<9) m_curr_img++;
		if (m_curr_img>9) m_curr_img--;
	}

	if (u || afterburn) {
		if (afterburn) {
			m_max_speed = 70.0f;
			m_dx += sin(m_turn_a*DEGtoRAD) * 0.8f;
			m_dy += -cos(m_turn_a*DEGtoRAD) * 0.8f;	
		} else {
			m_max_speed = 30.0f;
			m_dx += sin(m_turn_a*DEGtoRAD) * 0.4f;
			m_dy += -cos(m_turn_a*DEGtoRAD) * 0.4f;	
		}
	} else if (d) {
		m_dx -= sin(m_turn_a*DEGtoRAD) * 0.2f;
		m_dy -= -cos(m_turn_a*DEGtoRAD) * 0.2f;
	} else {
		m_speed *= 0.99;
		m_dx *= 0.99;
		m_dy *= 0.99;
	}
	m_speed = sqrt(m_dx*m_dx+m_dy*m_dy);
	if (m_speed > m_max_speed) {
		m_speed *= 0.9;
		m_dx *= 0.9; m_dy *= 0.9;
	}
	if (m_speed < -5.0f) {
		m_speed = -5.0f;
		m_dx *= 0.7; m_dy *= 0.7;
	}

}

void Ship::UpdateShipByData (float nx, float ny, float ndx, float ndy, float nv, float ns)
{
	m_x = nx;		m_y = ny;
	m_dx = ndx;		m_dy = ndy;
	m_vel_a = nv;	m_speed = ns;	
}

void Ship::UpdateShipByData (float ndt, float nx, float ny, float ndx, float ndy, float nva, float nsp, float nta, float ntda)
{
	m_dt = ndt;
	m_x = nx;
	m_y = ny;
	m_dx = ndx;
	m_dy = ndy;
	m_vel_a = nva;
	m_speed = nsp;	
	m_turn_a = nta;
	m_turn_da = ntda;	
	//m_dx = sin(m_vel_a*DEGtoRAD)*m_speed;
	//m_dy = cos(m_vel_a*DEGtoRAD)*m_speed;
}

void Ship::Move (void)
{
	m_turn_a += m_turn_da;									// Adjust angle by change-in-angle	
	if (m_turn_a < 0) m_turn_a += 360;
	if (m_turn_a > 360) m_turn_a -= 360;	
	m_x += m_dx;										// Move ship position by ship's velocity
	m_y += m_dy;
}

// Ship - Draw
// Draw the ship's shadow, target, and the ship itself
void Ship::Draw (float sx, float sy, int frame, Image* shield_imgs)
{
	float dist;
	float z = sin(frame*DEGtoRAD)*10.0 + 35.0;
	int centerx, centery;
	int xres = WinGL.GetWinX();
	int yres = WinGL.GetWinY();
	int pos_x, pos_y;

	centerx = xres/2;
	centery = yres/2;
	pos_x = int(m_x-sx+centerx);
	pos_y = int(m_y-sy+centery);

	WinGL.DrawImage (ships[SHIP_IMG_SHADOW], pos_x, pos_y+z, m_turn_a, 1.0f);

	WinGL.DrawImage (ships[m_curr_img], pos_x, pos_y, m_turn_a, 1.0f);

	if (m_shield_on>=0) {		
		WinGL.DrawImage (shield_imgs[m_shield_on], pos_x, pos_y, m_shield_ang, 1.0f);
		m_shield_on--;
	}

	float target_x, target_y;
	target_x = m_x - sx;
	target_y = m_y - sy;
	dist = (float) sqrt(target_x*target_x + target_y*target_y);
	if (dist > 50.0) {
		target_x *= 60.0f/dist;
		target_y *= 60.0f/dist;
		WinGL.DrawImage (target, (int) target_x + centerx,
			(int) target_y + centery, 0.0, 1.0f);
	} else {
		WinGL.DrawFill (10, yres-30, 110, yres-20, 0.0, 0.4, 0.0, 0.5);
		WinGL.DrawFill (xres-110, yres-30, xres-10, yres-20, 0.0, 0.4, 0.0, 0.5);
		WinGL.DrawFill (10, yres-30, 10+(m_health/10), yres-20, 0.7, 1.0, 0.7, 0.7);
		WinGL.DrawFill (xres-110, yres-30, xres-110+(m_shield/10), yres-20, 0.9, 1.0, 0.7, 0.7);
	}

	glColor4f (0.7, 0.8, 0.7, 1.0);
	WinGL.DrawText (pos_x + 50, pos_y, name);
	glColor4f (1.0, 1.0, 1.0, 1.0);
}
