
#include <windows.h>
#include <string.h>

#include "sound.h"

SoundX::SoundX()
{
	samples_per_sec = 0;
	bits_per_sample = 0;
	length = 0;
	num_bytes = 0;
	num_samples = 0;	
	data = NULL;
	dx_index = -1;
	dx_refresh = true;
}

SoundX::~SoundX()
{
}

bool SoundX::ReadName (File& wav, Buffer &code, char *match_name)
{
	unsigned char buf[5];
	wav.Read (4, buf);
	if (strncmp ((char*) buf, match_name, 4)==0)	return true;
	else											return false;
}

bool SoundX::ReadSize (File& wav, Buffer &code, unsigned int &size)
{
	XBYTE4 fourbytes;
	wav.Read4 (fourbytes); 
	size = code.ReadL (fourbytes);
	return true;
}

bool SoundX::LoadWav (char *filename)
{
	XBYTE2 twobytes;
	XBYTE4 fourbytes;
	XBYTE2 format;
	unsigned int size;
	int n;

	File wav;
	Buffer code;

	if (wav.Open (filename, FILE_READ | FILE_RANDOM) != FILE_EXIST_NO) {
		code.SetOrder (BUFFER_ORDER_LBF);
		
		if (ReadName(wav, code, "RIFF")==false) {wav.Close(); return false;}
		if (ReadSize(wav, code, size)==false)	{wav.Close(); return false;}
		if (ReadName(wav, code, "WAVE")==false)	{wav.Close(); return false;}	
		if (ReadName(wav, code, "fmt ")==false)	{wav.Close(); return false;}
		if (ReadSize(wav, code, size)==false)	{wav.Close(); return false;}
		if (size!=16)							{wav.Close(); return false;}
		// Read WAV Format Header
			wav.Read2 (twobytes); format = code.ReadI (twobytes);				// WAV Format Tag (1 = PCM) (IGNORED)
			if (format!=1) {wav.Close(); return false;}
			wav.Read2 (twobytes); num_channels = code.ReadI (twobytes);				// WAV Number of Channels
			wav.Read4 (fourbytes); samples_per_sec = code.ReadL (fourbytes);	// WAV Samples per Second
			wav.Read4 (fourbytes);												// WAV Avergae Bytes per Second (IGNORED)
			wav.Read2 (twobytes);												// WAV Block Align (IGNORED)
			wav.Read2 (twobytes); bits_per_sample = code.ReadI (twobytes);		// WAV Number of Channels
		if (ReadName(wav, code, "data")==false)	{wav.Close(); return false;}	
		if (ReadSize(wav, code, size)==false)	{wav.Close(); return false;}

		// Read WAV Data
		num_bytes = size;								// Load number of bytes in data (from file)				
		num_samples = num_bytes / ((bits_per_sample/8) * num_channels);	// Calculate number of samples
		length = (float) num_samples / (float) samples_per_sec;		// Calculate length of sound (in seconds)
		if (data!=NULL) {delete [] data; data = NULL;}
		data = new unsigned char[num_bytes];
		if (bits_per_sample <= 8) {
			for (n = 0; n < num_bytes; n+=num_channels) 
				wav.Read (num_channels, data + n);
		} else if (bits_per_sample <=16) {
			for (n = 0; n < num_bytes; n+=(num_channels<<1)) 
				wav.Read (num_channels<<1, data + n);
		}		
		dx_refresh = true;
		wav.Close ();
	} else {
		char disp[200];
		sprintf (disp, "Cannot find sound file: %s", filename);
		MessageBox (NULL, filename, "Sound Error", MB_OK);
		return false;
	}
	return true;
}

void SoundX::CutLength (float new_length)
{
	int new_samples, new_bytes;
	unsigned char* new_data;

	if (data!=NULL && length > new_length) {
		new_samples = (int) (samples_per_sec * new_length);
		new_bytes = (bits_per_sample/8) * num_channels * new_samples;	
		new_data = new unsigned char[new_bytes];
		memcpy (new_data, data, new_bytes);
		if (data!=NULL) {delete [] data; data = NULL;}
		data = new_data;
		num_bytes = new_bytes;
		num_samples = new_samples;
		length = new_length;
		dx_refresh = true;
	}
}

bool SoundX::NeedUpdate (void)
{
	if (dx_index==-1 || dx_refresh==true) return true;
	return false;
}