
#include "win-gl.h"

// WinMain and WindowFunc functions supplied by WindowsDX
LRESULT CALLBACK WindowFunc (HWND hWnd,	UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_ACTIVATEAPP: WinGL.SetActivation (wParam!=0); break;
	case WM_MOVE: break;
	case WM_CHAR: WinGL.SetKey ((TCHAR) wParam); break;
	case WM_KEYDOWN: WinGL.SetKeyDown ((TCHAR) wParam); break;		
	case WM_KEYUP: WinGL.SetKeyUp ((TCHAR) wParam); break;
	case WM_CREATE:	break;
	case WM_SETFOCUS: 
		if (WinGL.GetStatus()==WINGL_READY) WinGL.SetActivation (true);
		break;
	case WM_KILLFOCUS: WinGL.SetActivation(false); break;
	case WM_PAINT: break;
	case WM_SIZE: break;
	case WM_CLOSE: WinGL.Close(); DestroyWindow (hWnd); break;
	case WM_DESTROY: PostQuitMessage (0); break;
	case WM_MOUSEMOVE:		WinGL.SetMouseMove (LOWORD (lParam), HIWORD (lParam));	break;
	case WM_LBUTTONDOWN:	WinGL.SetMousePress (MK_LBUTTON);	break;
	case WM_MBUTTONDOWN:	WinGL.SetMousePress (MK_MBUTTON);	break;
	case WM_RBUTTONDOWN:	WinGL.SetMousePress (MK_RBUTTON);	break;
	case WM_RBUTTONUP: case WM_MBUTTONUP: case WM_LBUTTONUP:	WinGL.SetMouseRelease ();			break;
	}
	
	return DefWindowProc (hWnd, message, wParam, lParam);		
}

int WINAPI WinMain (HINSTANCE hThisInst, HINSTANCE hPrevInst, LPSTR lpszArgs, int nThisMode)
{
	MSG msg;
	
	WinGL.InitializeInstance (hThisInst, nThisMode);	// Initialize the application instance

	initialize ();										// Initialize WinGL via user
	
	if (WinGL.GetStatus() != WINGL_READY) {				// If initialization problem..
		WinGL.CloseAndReportProblem ();					// Close WinGL and report it
		return -1;
	}
	
	for (;;) {		
		if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE)) {
			if (msg.message == WM_QUIT) break;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		} 
		main();		
	}
	//WinGL.CleanUp ();
	return msg.wParam;
}

//==================================
// WINGL - INITIALIZATION FUNCTIONS
//==================================

WindowsGL::WindowsGL ()
{	
	win_status = WINGL_NO_INIT;
	win_keys = 0;
}

WindowsGL::~WindowsGL (void)
{
}

bool WindowsGL::Initialize (void)
{
	return Initialize (false, -1, -1, -1);
}

bool WindowsGL::Initialize (bool f, int x, int y, int b)
{
	win_request_full = f;
	win_request_bpp = b;
	win_request_xres = x;
	win_request_yres = y;

	if (InitializeClass()==false)	return false;
	if (InitializeDisplay()==false)	return false;
	if (InitializeWin()==false)		return false;
	if (InitializeContext()==false)	return false;
	if (InitializeFonts()==false)	return false;
	if (InitializeSound()==false)	return false;
	return true;
}

bool WindowsGL::InitializeInstance (HINSTANCE hThisInst, int nThisMode)
{
	win_hinst = hThisInst;
	win_hmode = nThisMode;
	return true;
}

bool WindowsGL::InitializeClass (void)
{
	WNDCLASS wcl;

  	wcl.hCursor			= LoadCursor(NULL,IDC_ARROW);
	wcl.hIcon			= NULL;
    wcl.lpszMenuName	= "Menu";
    wcl.lpszClassName	= "WinGL";
	wcl.hInstance		= win_hinst;
	wcl.style			= CS_VREDRAW | CS_HREDRAW;
    wcl.lpfnWndProc		= (WNDPROC) WindowFunc;
	wcl.cbClsExtra		= 0;
    wcl.cbWndExtra		= 0;
    wcl.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
    
    if (!RegisterClass(&wcl)) {
		win_status = WINGL_INITCLASS_FAILED;
		return false;
	}
	win_status = WINGL_INITCLASS_OK;
	return true;
}

bool WindowsGL::InitializeDisplay (void)
{
	DWORD style;
	HDC hdc;	
	
	// Get actual screen dimensions and bpp
	hdc = GetDC (NULL);									// Get device context for entire screen		
	win_original_bpp = GetDeviceCaps (hdc, BITSPIXEL);	// Get actual bpp
	win_original_xres = GetSystemMetrics(SM_CXSCREEN);	// Get actual xres
	win_original_yres = GetSystemMetrics(SM_CYSCREEN);	// Get actual yres	
	ReleaseDC (NULL, hdc);

	// Set requested settings to current screen settings if user did not specify any
	if (win_request_bpp!=8 && win_request_bpp!=16 && win_request_bpp!=24 && win_request_bpp!=32) win_request_bpp = win_original_bpp;
	if (win_request_xres<=0) win_request_xres = (win_original_xres*3)/4;
	if (win_request_yres<=0) win_request_yres = (win_original_yres*3)/4;	

	// Set first attempted screen dimensions and bpp	
	DEVMODE devMode;
	memset(&devMode, 0, sizeof(devMode));
	devMode.dmSize = sizeof(devMode);		
	if (win_request_full == true) {				// Requested full screen?
		style = CDS_FULLSCREEN;			
		win_full = true;						// Try full screen
		win_bpp = win_request_bpp;				// Try requested bpp first.
		win_xres = win_request_xres;			// Window dimensions should match 
		win_yres = win_request_yres;			// device dimensions in full screen mode
		devMode.dmBitsPerPel = win_request_bpp;	// Try requested bpp first.
		devMode.dmPelsWidth = win_request_xres; // Use requested dimensions as
		devMode.dmPelsHeight = win_request_yres; // potential device dimensions
		devMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
	} else {
		style = 0;
		win_full = false;						// Try in windowed mode
		win_bpp = win_request_bpp;				// Try requested bpp first.
		win_xres = win_request_xres;			// Window dimensions should be
		win_yres = win_request_yres;			// whatever was requested
		devMode.dmBitsPerPel = win_request_bpp;	// Try requested bpp first.
		devMode.dmPelsWidth = win_original_xres; // Device dimensions should be the 
		devMode.dmPelsHeight = win_original_yres; // original screen settings
		devMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
	}
	
	// Attempt to set display mode 
	// (Robust mode initialization method by R.C. Hoetzlein)
	for (int pass = 1; pass <= 4 && win_status!=WINGL_INITDISP_OK; pass++) {
		switch (pass) {
		case 1:	break;		// Pass 1 - Try requested mode with no changes
		case 2: {			// Pass 2 - Try requested mode but without full screen
			if (win_full==true) {win_full = false; style = 0;}
		} break;
		case 3: {			// Pass 3 - Try requested mode but with original screen bpp
			devMode.dmBitsPerPel = win_original_bpp;
			win_bpp = win_original_bpp;
		} break;
		case 4: {			// Pass 4 - Try original screen settings
			devMode.dmPelsWidth = win_original_xres; 
			devMode.dmPelsHeight = win_original_yres; 
			if (win_xres >= win_original_xres) win_xres = (win_original_xres * 3)/4;
			if (win_yres >= win_original_yres) win_yres = (win_original_yres * 3)/4;
		} break;	
		}
		if (ChangeDisplaySettings (&devMode, style)==DISP_CHANGE_SUCCESSFUL)
			win_status = WINGL_INITDISP_OK;
	}
	if (win_status==WINGL_INITDISP_OK) {
		return true;
	} else {
		win_status = WINGL_INITDISP_FAILED;
		return false;
	}	
}

bool WindowsGL::InitializeWin (void)
{
	DWORD	style_ex, style;
	RECT	rect;
	
	// Set window style and size based on accepted display settings
	if (win_full==true) {
		style_ex = WS_EX_APPWINDOW;
		style = WS_POPUP;
		rect.left = 0; rect.top = 0;		
		ShowCursor (false);
	} else {
		style_ex = WS_EX_APPWINDOW;
		style = WS_OVERLAPPED | WS_VISIBLE | WS_SYSMENU; // | WS_CLIPCHILDREN | WS_CLIPSIBLINGS;
		rect.left = (win_original_xres/2) - (win_xres/2);
		rect.top = (win_original_yres/2) - (win_yres/2);
	}
	rect.right = rect.left + win_xres;		
	rect.bottom = rect.top + win_yres;
	
	// Attempt to create a window
	for (int pass = 1; pass <= 2 && win_status!=WINGL_INITWIN_OK; pass++) {
		switch (pass) {
		case 1: AdjustWindowRectEx (&rect, style, false, style_ex); break;  
		case 2: break;
		}
		win_hwnd = CreateWindowEx(style_ex, "WinGL", "", style, rect.left, rect.top, rect.right-rect.left, rect.bottom-rect.top, 0, 0, win_hinst, 0);
		if (win_hwnd!=NULL) win_status = WINGL_INITWIN_OK;
	}

	if (win_status==WINGL_INITWIN_OK) {		
		ShowWindow (win_hwnd, SW_SHOW);
		if (UpdateWindow (win_hwnd)==false) {
			win_status = WINGL_SHOWWIN_FAILED;
			return false;
		}
		win_status = WINGL_SHOWWIN_OK;
		return true;
	} else {
		win_status = WINGL_INITWIN_FAILED;
		return false;
	}	
}

bool WindowsGL::InitializeContext (void)
{
	int nPixelFormat;
	int nResult;
	
	static PIXELFORMATDESCRIPTOR pfd = {
		sizeof (PIXELFORMATDESCRIPTOR),
		1,
		PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
		PFD_TYPE_RGBA,
		32,
		0,0,0,0,0,0,
		0,0,0,
		0,0,0,0,
		16,
		0,0,
		PFD_MAIN_PLANE,
		0,
		0,0,0};

	win_hdc = GetDC (win_hwnd);

	nPixelFormat = ChoosePixelFormat (win_hdc, &pfd);
	if (nPixelFormat==0) {win_status = WINGL_INITCHOOSEFORMAT_FAILED; return false;}

	nResult = SetPixelFormat (win_hdc, nPixelFormat, &pfd);
	if (nResult==false) {win_status = WINGL_INITSETFORMAT_FAILED; return false;}
	
	win_hrc = wglCreateContext (win_hdc);
	if (win_hrc==NULL) {win_status = WINGL_CREATECONTEXT_FAILED; return false;}

	nResult = wglMakeCurrent (win_hdc, win_hrc);
	if (nResult==false) {win_status = WINGL_MAKECONTEXT_FAILED; return false;}

	glPixelStorei (GL_UNPACK_ALIGNMENT, 1);

	return true;
}

bool WindowsGL::InitializeFonts (void)
{
	HFONT hFont;

	win_gl_font = glGenLists (96);

	hFont = CreateFont (16, 0, 0, 0, FW_BOLD, false, false, false, ANSI_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, ANTIALIASED_QUALITY, FF_DONTCARE | DEFAULT_PITCH, "System");

	if (!hFont) return false;

	SelectObject (win_hdc, hFont);
	
	wglUseFontBitmaps (win_hdc, 32, 96, win_gl_font);

	win_status = WINGL_READY;

	return true;
}

bool WindowsGL::InitializeSound (void)
{
	if (DS_OK==DirectSoundCreate(NULL, &ds_main, NULL)) {
		if (DS_OK!=IDirectSound_SetCooperativeLevel(ds_main, win_hwnd, DSSCL_NORMAL)) {
			IDirectSound_Release(ds_main);
			win_status = WINGL_SOUNDCOOP_FAILED;
			sound_support = false;
			return false;
		}
	} else {		
		win_status = WINGL_SOUNDINIT_FAILED;
		sound_support = false;
        return false;
    }
	sound_support = true;
	return true;	
}


bool WindowsGL::ResetDisplay (void)
{	
	DEVMODE devMode;
	memset(&devMode, 0, sizeof(devMode));
	devMode.dmSize = sizeof(devMode);
	devMode.dmPelsWidth = win_original_xres;
	devMode.dmPelsHeight = win_original_yres;
	devMode.dmBitsPerPel = win_original_bpp;
	devMode.dmFields = DM_PELSWIDTH | DM_PELSHEIGHT | DM_BITSPERPEL;
	
	ChangeDisplaySettings (&devMode, 0);
	return true;
}

void WindowsGL::CloseAndReportProblem (int p)
{
	win_status = p;
	CloseAndReportProblem ();
}

void WindowsGL::CloseAndReportProblem (void)
{
	char error[100];
	char recommend[300];	
	char disp[1000];

	ResetDisplay ();
	
	switch (win_status) {
	case WINGL_NO_INIT: {
		sprintf (error, "Error: NO_INIT (User did not initialize WinGL)");
		sprintf (recommend, "You must start by calling:\n  WinGL.Initialize (fullscreen, xres, yres, bpp)\n\nfrom the initialize() function. See documentation for starting WinGL.");
	} break;
	case WINGL_INITCLASS_FAILED: {
		sprintf (error, "Error: INITCLASS_FAILED (Cannot Initialize Window Class)");
		sprintf (recommend, "WinGL was unable to create windows class. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;
	case WINGL_INITDISP_FAILED: {
		sprintf (error, "Error: INITDISP_FAILED (Cannot Initialize Display)");
		sprintf (recommend, "WinGL was unable to initialize the display. Try initializing with a different screen\nresolution and/or bpp using WinGL.Initialize(fullscreen, xres, yres, bpp).\nIf in doubt, use WinGL.Initialize()");
	} break;
	case WINGL_INITWIN_FAILED: {
		sprintf (error, "Error: INITWIN_FAILED (Cannot Initialize Main Window)");
		sprintf (recommend, "WinGL was unable to create primary window. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;
	case WINGL_SHOWWIN_FAILED: {
		sprintf (error, "Error: SHOWWIN_FAILED (Unable to show Main Window)");
		sprintf (recommend, "WinGL was unable to show the primary window. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;
	case WINGL_INITCHOOSEFORMAT_FAILED: {
		sprintf (error, "Error: INITCHOOSEFORMAT_FAILED (Unable to select Pixel Format)");
		sprintf (recommend, "WinGL was unable to create a pixel format. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;
	case WINGL_INITSETFORMAT_FAILED: {
		sprintf (error, "Error: INITSETFORMAT_FAILED (Unable to set Pixel Format)");
		sprintf (recommend, "WinGL was unable to set the pixel format. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;
	case WINGL_CREATECONTEXT_FAILED: {
		sprintf (error, "Error: CREATECONTEXT_FAILED (Unable to create OpenGL Context)");
		sprintf (recommend, "WinGL was unable to create an OpenGL Context. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;
	case WINGL_MAKECONTEXT_FAILED: {
		sprintf (error, "Error: MAKECONTEXT_FAILED (Unable to make OpenGL Context current)");
		sprintf (recommend, "WinGL was unable to make the OpenGL Context current. Are you using Windows 98/2000/NT? WinGL requires one of these.\nStill having trouble? Please report error and system configuration.");
	} break;	
	case WINGL_SOUNDINIT_FAILED: {
		sprintf (error, "Error: SOUNDINIT_FAILED (Unable to initialize Direct Sound)");
		sprintf (recommend, "GameX unable to initialize sound. Is sound card installed and working?");
	} break;
	case WINGL_SOUNDCOOP_FAILED: {
		sprintf (error, "Error: SOUNDCOOP_FAILED (Unable to set Direct Sound Cooperative Level");
		sprintf (recommend, "GameX unable to set Direct Sound cooperative level. Is sound card installed and working?");
	} break;	
	case WINGL_SOUNDCREATE_FAILED: {
		sprintf (error, "Error: SOUNDCREATE_FAILED (Unable to create Direct Sound buffer.");
		sprintf (recommend, "GameX unable to create a sound buffer for your sound. Try shortening the sound file (total length), reducing the sound resolution (bits per sample), reducing the sound quality (samples per second), or eliminating a channel (mono vs. stereo).");
	} break;
	case WINGL_SOUNDMATCH_FAILED: {
		sprintf (error, "Error: SOUNDMATCH_FAILED (Loaded sounds do not match.");
		sprintf (recommend, "You are attempting to load several sounds with different quality, resolution, and channel info. All sounds to be played simultaneously must have the same resolution (bits per sample), same quality (samples per second), and same number of channels (mono vs. stereo). Edit WAV files accordinging.");
	} break;
	case WINGL_SOUNDCAPS_FAILED: {
		sprintf (error, "Error: SOUNDCAPS_FAILED (Unable to get Sound Buffer Capabilities.");
		sprintf (recommend, "GameX was unable to retrieve sound buffer capabilities. You may be running out of memory. Close any open applications. Make sure you have DirectX 3.1, 7 or 8.1.");
	} break;
	case WINGL_SOUNDBUFLEN_FAILED: {
		sprintf (error, "Error: SOUNDBUFLEN_FAILED (Direct Sound buffer and SoundX buffers do not match.");
		sprintf (recommend, "Direct Sound and SoundX buffer lengths do not match. Internal error. Please report this problem and the code that generated it.");	
	} break;
	case WINGL_SOUNDLOST_FAIL: {
		sprintf (error, "Error: SOUNDLOST_FAIL (Direct Sound buffer was lost.");
		sprintf (recommend, "Direct Sound buffer was lost. Make sure you are using DirectX 3.1, 7 or 8.1. Close any open applications. Try restarting system. If you are using a large or high quality sound file, try reducing it.");
	} break;
	case WINGL_SOUNDINVALID_FAIL: {
		sprintf (error, "Error: SOUNDINVALID_FAIL (Direct Sound Invalid Call.");
		sprintf (recommend, "DirectX, via GameX, made an invalid sound call. Make sure you are using DirectX 3.1, 7 or 8.1. Close any open applications. Try restarting system. If you are using a large or high quality sound file, try reducing it.");
	} break;
	case WINGL_SOUNDPARAM_FAIL: {
		sprintf (error, "Error: SOUNDPARAM_FAIL (Direct Sound Invalid Parameter.");
		sprintf (recommend, "DirectX, via GameX, used an invalid parameter. Make sure you are using DirectX 3.1, 7 or 8.1. Close any open applications. Try restarting system. If you are using a large or high quality sound file, try reducing it.");
	} break;
	case WINGL_SOUNDPRIOLEVEL_FAIL: {
		sprintf (error, "Error: SOUNDPARAM_FAIL (Direct Sound needs Priority Level");
		sprintf (recommend, "DirectX, via GameX, is using an invalid priority level. Make sure you are using DirectX 3.1, 7 or 8.1. Close any open applications. Try restarting system. If you are using a large or high quality sound file, try reducing it.");		
	} break;	


	default: {
		sprintf (error, "Error: UNREGISTERED");
		sprintf (recommend, "Unregistered error with WinGL. Please report.");
	} break;
	}
	sprintf (disp, "Unable to start WinGL.\n\n%s\n\n%s", error, recommend);	
	MessageBox (NULL, disp, "WinGL", MB_OK);	
	
	sprintf (disp, "WINGL %s\n", error);
	//debug.Output (disp, "");
}

//============================
// WINGL - KEYBOARD FUNCTIONS 
//============================

void WindowsGL::SetKey (char ch)
{
	win_keybuf[win_keys++] = ch; 
	if (win_keys > WINGL_KEY_BUFSIZE) {
		win_keys = WINGL_KEY_BUFSIZE;
		//debug.Output ("WinDX: Keyboard buffer overflow.\n", "");
	}
}

char WindowsGL::GetKey (void) 
{	
	if (win_keys>0) {
		win_keys--;		
		return win_keybuf[win_keys];
    } else {
		return 0;
	}	
}

void WindowsGL::DrawOpen2D (void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	
	glShadeModel (GL_FLAT);	
	
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();	
	glMatrixMode (GL_MODELVIEW);	
	glLoadIdentity();
	glViewport (0, 0, win_xres, win_yres);
	glOrtho (0, win_xres, 0, win_yres, -200.0, 200.0);
	glEnable (GL_TEXTURE_2D);
	
	glEnable (GL_BLEND); 
	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
	
	glDisable (GL_DEPTH_TEST);
	
	glBegin (GL_QUADS);
	glColor3f (1.0, 1.0, 1.0);
	glVertex2f (-5000.0f, -5000.0f);
	glVertex2f (5000.0f, -5000.0f);
	glVertex2f (5000.0f, 5000.0f);
	glVertex2f (-5000.0f, 5000.0f);
	glEnd ();
}

//============================
// WINGL - IMAGE FUNCTIONS 
//============================

#ifdef USE_IMAGE

void WindowsGL::DrawImageDirect (Image& img, int x, int y)
{
	glDisable (GL_TEXTURE_2D);
	glLoadIdentity ();
	glScalef (2.0f/win_xres, -(2.0f/win_yres), 0.0f);
	glPixelZoom (1.0, -1.0);
	glTranslatef (-win_xres/2.0f, -win_yres/2.0f, 0.0f);		
	glRasterPos2i (x, -y);	
	glDrawPixels (img.GetWidth(), img.GetHeight(), img.GetGLFormat(), GL_UNSIGNED_BYTE, img.GetData());
}

void WindowsGL::DrawImage (Image& img, int x, int y, float ang, float scale)
{
	if (img.GetStatus()==IMG_NOT_READY) AddImageObject (img);
	//if (img.GetStatus()==IMG_UPDATED) UpdateImageObject (
		
	glCallList (win_lists[img.GetGLList()]);
	
	glTranslatef ((float) x, (float) y, 0.0f);
	glRotatef (ang, 0.0, 0.0, 1.0);	
	glScalef (scale, scale, 0.0);
	glTranslatef (-img.GetPivotX(), -img.GetPivotY(), 0.0f);

	glCallList (win_lists[img.GetGLList()]+1);
}

void WindowsGL::DrawBackground (Image& img, int x, int y)
{
	int xres, yres;
	int map_x1, map_y1;
	int map_x2, map_y2;	
	int map_numx, map_numy;	
	int off_x, off_y;
	int img_x, img_y;	
	int mx, my;

	xres = img.GetSubWidth(); yres = img.GetSubHeight();
	ClipTile (x+win_xres, y+win_yres, xres, yres, map_x2, map_y2, off_x, off_y);
	ClipTile (x, y, xres, yres, map_x1, map_y1, off_x, off_y);

	map_numx = map_x2 - map_x1 + 1;
	map_numy = map_y2 - map_y1 + 1; 	
	for (my = 0; my < map_numy; my++) {
		img_y = my * yres - off_y;
		img_x = -off_x;
		for (mx = 0; mx < map_numx; mx++) {
			DrawImage (img, img_x, img_y, 0.0f, 1.0f);
			img_x += xres;
		}
	}
}


void WindowsGL::DrawColor (int r, int g, int b)
{
	glColor3f (r/255.0f, g/255.0f, b/255.0f);
}

void WindowsGL::DrawText (int x, int y, char* msg)
{
	glDisable (GL_TEXTURE_2D);
	glLoadIdentity ();
	glScalef (2.0f/win_xres, -(2.0f/win_yres), 0.0f);
	glPixelZoom (1.0, -1.0);
	glTranslatef (-win_xres/2.0f, -win_yres/2.0f, 0.0f);		
	glRasterPos2i (x, y+10);
	glPushAttrib (GL_LIST_BIT);
		glListBase (win_gl_font - 32);
		glCallLists (strlen(msg), GL_UNSIGNED_BYTE, msg);		
	glPopAttrib ();
	glEnable (GL_TEXTURE_2D);
}

void WindowsGL::DrawFill (int x1, int y1, int x2, int y2, float r, float g, float b, float a)
{
	glDisable (GL_TEXTURE_2D);	
	glLoadIdentity ();
	glScalef (2.0f/win_xres, -(2.0f/win_yres), 0.0f);
	glPixelZoom (1.0, -1.0);
	glTranslatef (-win_xres/2.0f, -win_yres/2.0f, 0.0f);		
	glColor4f (r, g, b, a);
	
	glBegin (GL_QUADS);
	glVertex2f ((float) x1 , (float) y1);
	glVertex2f ((float) x2 , (float) y1);
	glVertex2f ((float) x2 , (float) y2);
	glVertex2f ((float) x1 , (float) y2);
	glEnd ();

	glColor4f (1.0, 1.0, 1.0, 1.0);
	glEnable (GL_TEXTURE_2D);
	
}

int xres = WinGL.GetWinX();
	int yres = WinGL.GetWinY();


int WindowsGL::AddTexture (Image& img)
{
	if (win_num_textures==win_max_textures) {
		if (win_max_textures==0) win_max_textures = 1; else win_max_textures <<= 1;
		GLuint* new_textures = new GLuint[win_max_textures];
		memcpy (new_textures, win_textures, sizeof(GLuint)*win_num_textures);
		if (win_textures!=NULL) delete [] win_textures;
		win_textures = new_textures;
	}
	glGenTextures (1, &win_textures[win_num_textures]);
	glBindTexture (GL_TEXTURE_2D, win_textures[win_num_textures]);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	int c = 3;
	if (img.GetGLFormat()==GL_RGBA) c = 4;	
	glTexImage2D (GL_TEXTURE_2D, 0, c, img.GetWidth(), img.GetHeight(), 0, img.GetGLFormat(), GL_UNSIGNED_BYTE, img.GetData());	
	win_num_textures++;
	return win_num_textures-1;
}

int WindowsGL::AddList (void)
{
	if (win_num_lists==win_max_lists) {
		if (win_max_lists==0) win_max_lists = 1; else win_max_lists <<= 1;
		GLuint* new_lists = new GLuint[win_max_lists];
		memcpy (new_lists, win_lists, sizeof(GLuint)*win_num_lists);
		if (win_lists!=NULL) delete [] win_lists;
		win_lists = new_lists;
	}
	win_lists[win_num_lists] = glGenLists (2);
	win_num_lists++;
	return win_num_lists-1;
}

void WindowsGL::Close (void)
{
	int n;
	for (n=0; n < win_num_textures; n++) 
		glDeleteTextures (1, &win_textures[n]);

	for (n=0; n < win_num_lists; n++) 
		glDeleteLists (win_lists[n], 2);
}

void WindowsGL::AddImageObject (Image& img)
{
	float sx, sy;

	img.ConvertToPowerImage ();	
	img.SetGLObject (AddTexture (img));
	img.SetGLList (AddList());
	
	glNewList (win_lists[img.GetGLList()], GL_COMPILE);		
		glLoadIdentity();
		glScalef (2.0f/win_xres, -(2.0f/win_yres), 0.0f);
		glTranslatef (-win_xres/2.0f, -win_yres/2.0f, 0.0f);		
		glBindTexture (GL_TEXTURE_2D, win_textures[img.GetGLObject()]);
	glEndList();
	
	glNewList (win_lists[img.GetGLList()]+1, GL_COMPILE);			
		glBegin (GL_QUADS);
		sx = img.GetSubX(); sy = img.GetSubY();		
		glTexCoord2f (0.0, 0.0);	glVertex2f (0.0 , 0.0);
		glTexCoord2f (sx, 0.0);		glVertex2f ((float) img.GetSubWidth(), 0.0);
		glTexCoord2f (sx, sy);		glVertex2f ((float) img.GetSubWidth(), img.GetSubHeight());
		glTexCoord2f (0.0, sy);		glVertex2f (0.0, (float) img.GetSubHeight());
		glEnd ();
	glEndList();
}

void WindowsGL::PlaySound (SoundX* snd, int how)
{		
	if (sound_support) {
		if (snd->NeedUpdate()==true) UpdateSound (snd);	
		int num = snd->GetIndex();
	
		if (how==PLAY_REWIND) ds_sound_list[num]->SetCurrentPosition (0);	
		ds_sound_list[num]->Play (0,0,0);
	}
}

void WindowsGL::PlaySound (SoundX* snd, int how, float vol, float pan)
{		
	if (sound_support) {
		if (snd->NeedUpdate()==true) UpdateSound (snd);	
		int num = snd->GetIndex();
	
		if (how==PLAY_REWIND) ds_sound_list[num]->SetCurrentPosition (0);
		ds_sound_list[num]->SetVolume ((long) (-5000.0 + vol*5000.0));
		ds_sound_list[num]->SetPan ((long) (pan*10000.0));
		ds_sound_list[num]->Play (0,0,0);
	}
}

void WindowsGL::RewindSound (SoundX* snd)
{
	if (sound_support) {
		if (snd->NeedUpdate()==true) UpdateSound (snd);	
		int num = snd->GetIndex();	
		ds_sound_list[num]->SetCurrentPosition(0);
	}
}


void WindowsGL::StopSound (SoundX* snd)
{
	if (sound_support) {
		int num = snd->GetIndex();
		if (num>=0) ds_sound_list[num]->Stop ();
	}
}

void WindowsGL::UpdateSound (SoundX* snd)
{
	signed short *buf_a, *buf_b;	
	unsigned long bytes_a, bytes_b;	

	int num_samples, num_channels;
	int result, num;
	int n, c;
	
	// Determine Direct Sound buffer to use based on SoundX object.
	// (If Direct Sound buffer does not exist for SoundX object, create one)
	num = snd->GetIndex();
	if (num==-1) {
		num = AddSound (snd);
		snd->SetIndex (num);
	}
	
	// Get requested Direct Sound buffer capabilities
	DSBCAPS caps;
	ZeroMemory(&caps, sizeof(caps));
	caps.dwSize = sizeof(caps);
	if (DS_OK != ds_sound_list[num]->GetCaps (&caps)) CloseAndReportProblem(WINGL_SOUNDCAPS_FAILED);
			
	// Get and check number of samples (SoundX sample count should match DS buffer count)
	num_channels = snd->GetNumChannels();
	num_samples = caps.dwBufferBytes / num_channels;
	if (num_samples!=snd->GetNumSamples()) CloseAndReportProblem(WINGL_SOUNDBUFLEN_FAILED);
	
	// Lock Direct Sound buffer
	result = ds_sound_list[num]->Lock (0, caps.dwBufferBytes, (void**) &buf_a, &bytes_a, (void**) &buf_b, &bytes_b, 0);
	n = 0;
	while (result==DSERR_BUFFERLOST && n<10) {
        ds_sound_list[num]->Restore();
        result = ds_sound_list[num]->Lock (0, caps.dwBufferBytes, (void**) &buf_a, &bytes_a, (void**) &buf_b, &bytes_b, 0);
		n++;
    }
	// Check for possible buffer errors
	if (DS_OK!=result) {		
		if (result==DSERR_BUFFERLOST)			win_status = WINGL_SOUNDLOST_FAIL;	
		else if (result==DSERR_INVALIDCALL)		win_status = WINGL_SOUNDINVALID_FAIL;
		else if (result==DSERR_INVALIDPARAM)	win_status = WINGL_SOUNDPARAM_FAIL;
		else if (result==DSERR_PRIOLEVELNEEDED)	win_status = WINGL_SOUNDPRIOLEVEL_FAIL;
		CloseAndReportProblem();
	}
	// Copy SoundX data to Direct Sound buffer
	if (snd->GetBPS()<=8) {
		unsigned char *data = (unsigned char*) snd->GetData();
		unsigned char *curr_a = (unsigned char*) buf_a;
		for (n = 0; n < num_samples; n++)
			for (c=0; c < num_channels; c++)
				*curr_a++ = *data++;
	} else {
		signed short *data = (signed short*) snd->GetData();
		signed short *curr_a = buf_a;
		curr_a = buf_a;
		for (n = 0; n < num_samples; n++)
			for (c=0; c < num_channels; c++) 
				*curr_a++ = *data++;			
	}
	
	// Unlock Direct Sound buffer
	ds_sound_list[num]->Unlock (&buf_a, bytes_a, &buf_b, bytes_b);

	snd->SetRefresh (false);
}

int WindowsGL::AddSound (SoundX* snd)
{
	SoundXPtr*				new_list;
	LPDIRECTSOUNDBUFFER*	new_ds_list;
	
	if (num_sounds==max_sounds) {
		max_sounds += 10;
		new_list = new SoundXPtr[max_sounds];
		new_ds_list = new LPDIRECTSOUNDBUFFER[max_sounds];
		memcpy (new_list, sound_list, sizeof(SoundXPtr)*num_sounds);
		memcpy (new_ds_list, ds_sound_list, sizeof(LPDIRECTSOUNDBUFFER)*num_sounds);
		if (sound_list!=NULL) {delete [] sound_list;}
		if (ds_sound_list!=NULL) {delete [] ds_sound_list;}
		sound_list = new_list;
		ds_sound_list = new_ds_list;
	}	
	sound_list[num_sounds] = snd;
	
	if (num_sounds==0) {
		ds_num_channels = snd->GetNumChannels();
		ds_bits_per_sample = snd->GetBPS();
		ds_samples_per_sec = snd->GetSamplesPerSec();
	} else {
		if (snd->GetNumChannels()!=ds_num_channels) CloseAndReportProblem (WINGL_SOUNDMATCH_FAILED);
		if (snd->GetBPS()!=ds_bits_per_sample) CloseAndReportProblem (WINGL_SOUNDMATCH_FAILED);
		if (snd->GetSamplesPerSec()!=ds_samples_per_sec) CloseAndReportProblem (WINGL_SOUNDMATCH_FAILED);
	}

	ds_sound_list[num_sounds] = NULL;
	
	WAVEFORMATEX format; 
	ZeroMemory(&format,sizeof(format)); 		
	format.wFormatTag = WAVE_FORMAT_PCM;
	format.nChannels = ds_num_channels;
	format.nSamplesPerSec = ds_samples_per_sec;
	format.wBitsPerSample = ds_bits_per_sample;
	format.nBlockAlign = (format.wBitsPerSample / 8) * format.nChannels; 
	format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign; 	
	
	DSBUFFERDESC bufferDesc; 
	ZeroMemory(&bufferDesc,sizeof(bufferDesc)); 
	bufferDesc.dwSize = sizeof(bufferDesc); 
	bufferDesc.dwFlags = DSBCAPS_CTRLFREQUENCY | DSBCAPS_CTRLPAN | DSBCAPS_CTRLVOLUME; 
	bufferDesc.dwBufferBytes = snd->GetNumSamples() * format.nBlockAlign;
	bufferDesc.lpwfxFormat = (LPWAVEFORMATEX) &format;

	if (FAILED (ds_main->CreateSoundBuffer(&bufferDesc, &ds_sound_list[num_sounds], NULL))) {		
		CloseAndReportProblem (WINGL_SOUNDCREATE_FAILED);
	}
	
	num_sounds++;
	return num_sounds-1;
}


static bool ClipTile (int x, int y, int xres, int yres, int &mx, int &my, int &offx, int &offy)
{
	if (x<0) {
		mx = ((int) (x / xres)) - 1;
		offx = (x % xres) + xres;
	} else {
		mx = ((int) (x / xres));
		offx = x % xres;
	}
	if (y<0) {
		my = ((int) (y / yres)) - 1;		
		offy = (y % yres) + yres;
	} else {
		my = ((int) (y / yres));		
		offy = y % yres;
	}	
	return true;
}


#endif
