


#ifndef WINGL_DEF
	#define WINGL_DEF

	#include <windows.h>
	#include <stdio.h>
	#include <gl/gl.h>
	#include <dsound.h>

	#include "common.h"
	
	#ifdef USE_SOUND
		#include "sound.h"
	#endif
	#ifdef USE_IMAGE
		#include "image.h"
	#endif	
	
	#define WINGL_NO_INIT					0
	#define WINGL_INITCLASS_FAILED			1
	#define WINGL_INITCLASS_OK				2
	#define WINGL_INITDISP_FAILED			3
	#define WINGL_INITDISP_OK				4
	#define WINGL_INITWIN_FAILED			5
	#define WINGL_INITWIN_OK				6
	#define WINGL_SHOWWIN_FAILED			7
	#define WINGL_SHOWWIN_OK				8
	#define WINGL_INITCHOOSEFORMAT_FAILED	9
	#define WINGL_INITSETFORMAT_FAILED		10
	#define WINGL_CREATECONTEXT_FAILED		11
	#define WINGL_MAKECONTEXT_FAILED		12
	#define WINGL_READY						13

	#define WINGL_SOUNDINIT_FAILED			14
	#define WINGL_SOUNDCOOP_FAILED			15
	#define WINGL_SOUNDCAPS_FAILED			16	
	#define WINGL_SOUNDBUFLEN_FAILED		17
	#define WINGL_SOUNDLOST_FAIL			18
	#define WINGL_SOUNDINVALID_FAIL			19
	#define WINGL_SOUNDPARAM_FAIL			20
	#define WINGL_SOUNDPRIOLEVEL_FAIL		21	
	#define WINGL_SOUNDMATCH_FAILED			22
	#define WINGL_SOUNDCREATE_FAILED		23	

	#define WINGL_KEY_BUFSIZE				1000
	
	class WindowsGL {
	public:
		WindowsGL ();
		~WindowsGL (void);		

		// Initialization Functions
		bool InitializeInstance (HINSTANCE hThisInst, int nThisMode);
		bool Initialize (void);
		bool Initialize (bool f, int x, int y, int bpp);
		bool InitializeClass (void);
		bool InitializeDisplay (void);
		bool InitializeWin (void);
		bool InitializeContext (void);
		bool InitializeFonts (void);
		bool InitializeSound (void);

		// General State Functions		
		void DrawOpen2D (void);
		inline void DrawClose (void)	{SwapBuffers (win_hdc);}
		void DrawColor (int r, int g, int b);
		void DrawText (int x, int y, char *msg);
		inline void SetBackColor (float r, float g, float b)	{glClearColor (r,g,b, 0);}		
		inline void SetActivation (bool act)	{win_active = act;}		
		inline int	GetStatus (void)			{return win_status;}
		bool ResetDisplay (void);
		void Close (void);
		void CloseAndReportProblem (void);
		void CloseAndReportProblem (int p);

		// Draw Images
		#ifdef USE_IMAGE
			int AddTexture (Image& img);
			int AddList (void);
			void AddImageObject (Image& img);
			void DrawImageDirect (Image& img, int x, int y);
			void DrawImage (Image& img, int x, int y, float ang, float scale);
			void DrawBackground (Image& img, int x, int y);
			void DrawFill (int x1, int y1, int x2, int y2, float r, float g, float b, float a);
		#endif

		// Play Sounds
		#ifdef USE_SOUND
			int AddSound (SoundX* snd);
			void UpdateSound (SoundX* snd);
			void PlaySound (SoundX* snd, int how);
			void PlaySound (SoundX* snd, int how, float vol, float pan);
			void StopSound (SoundX* snd);
			void RewindSound (SoundX* snd);
		#endif

		// Keyboard Functions
		void SetKey (char ch);
		char GetKey (void);
		inline void SetKeyUp (char ch)			{win_keydown[ch] = false;}
		inline void SetKeyDown (char ch)		{win_keydown[ch] = true;}		
		inline bool GetKeyDown (int n)			{return win_keydown[n];}

		// Mouse Functions		
		inline void SetMouseMove (int x, int y) {win_mouse_xl = win_mouse_x; win_mouse_yl = win_mouse_y; win_mouse_x = x; win_mouse_y = y;	win_mouse_motion = true;}
		inline void SetMousePress (int button)	{win_mouse_down = true; win_mouse_button = button;}
		inline void SetMouseRelease ()			{win_mouse_down = false; win_mouse_button = false;}
		inline int GetMouseX ()					{return win_mouse_x;}
		inline int GetMouseY ()					{return win_mouse_y;}

		inline int GetWinX()					{return win_xres;}
		inline int GetWinY()					{return win_yres;}
		// Status Functions
		HWND GetWindow (void)					{return win_hwnd;}
		
	private:		
		int			win_status;					// WindowsGL status (see defines)
		bool		win_active;

		int			win_original_bpp;			// Current screen settigs
		int			win_original_xres;
		int			win_original_yres;
		bool		win_request_full;			// Requested screen settings
		int			win_request_bpp;
		int			win_request_xres;
		int			win_request_yres;
		bool		win_full;					// Final screen settings
		int			win_bpp;
		int			win_xres;
		int			win_yres;

		HINSTANCE	win_hinst;					// WinGL Instance and Window
		HWND		win_hwnd;
		int			win_hmode;
		HDC			win_hdc;					// WinGL Device Context and Rendering Context
		HGLRC		win_hrc;

		int			win_keys;
		char		win_keybuf[WINGL_KEY_BUFSIZE];	// WinGL Keyboard Buffers
		bool		win_keydown[256];

		int			win_mouse_x, win_mouse_y;		// WinGL Mouse Parameters
		int			win_mouse_xl, win_mouse_yl;
		int			win_mouse_down, win_mouse_button;
		int			win_mouse_motion;
		
		int			win_rgb;
		GLuint		win_gl_font;

		#ifdef USE_IMAGE
			int			win_num_textures, win_max_textures;				
			GLuint*		win_textures;
			int			win_num_lists, win_max_lists;
			GLuint*		win_lists;
			float		win_img_depth;
		#endif

		#ifdef USE_SOUND
			LPDIRECTSOUND			ds_main;
			int						ds_samples_per_sec;
			int						ds_bits_per_sample;
			int						ds_num_channels;
			bool					sound_support;
			int						num_sounds;
			int						max_sounds;
			int						num_playing;
			SoundXPtr*				sound_list;
			LPDIRECTSOUNDBUFFER*	ds_sound_list;
		#endif
	};

	#ifdef WINGL_MAIN
		WindowsGL WinGL;		
	#else
		extern WindowsGL WinGL;	
		extern void initialize (void);
		extern void main (void);
	#endif

#endif


